/*
	file:
		Custom.cpp
	desc:
		This file contains the good stuff!
*/

// Includes
#include "Custom.h"
#include "GameScripting.h"
#include "TXD.h"

// Defines
#define SAFE_RELEASE(p) { if ( (p) ) { (p)->Release(); (p) = 0; } }

// Externals
extern HINSTANCE hInst;
extern DWORD SCRIPT_BASE;
extern GameGetObject_t GameGetObject;
extern GameGetVehicle_t GameGetVehicle;
extern GameGetActor_t GameGetActor;

// ViceVars
DWORD* dwCanPlayRadio	= (DWORD*)(0x978668);	// Vice City boolean - Can the radio play? (in-game)
BYTE* bInMenu			= (BYTE*)(0x869668);	// Vice City boolean - Are we in Menu?

DWORD* PLAYER_CHAR;		// Set in CustomInitialize() after
DWORD* PLAYER_ACTOR;	// SCRIPT_BASE is possibly changed.
DWORD* dwPlrPointer		= (DWORD*)0x94AD28;		// VC 1.0

// Globals
bool bOneTimeInit = false;			// One-time initialization flag.
float fScreenWidth;					// Screen width.
float fScreenHeight;				// Screen height.

ID3DXSprite* pHudSprite;			// Hud sprite.
IDirect3DTexture8* pHudTex;			// Hud texture.
IDirect3DTexture8* pWantedTex;		// Wanted texture.
IDirect3DTexture8* pMiscTex;		// Wanted texture.
IDirect3DTexture8* pAmmoTex;		// texture for displaying ammo.
IDirect3DTexture8* pWeaponsTex[34];	// weapons textures

DWORD* pMiscHudTex;

D3DXVECTOR2 vScaling;
D3DXVECTOR2 vScaling2;
D3DXVECTOR2 vTranslation;
D3DXVECTOR2 vTranslationWanted;
D3DXVECTOR2 vTranslationAmmo;

#define BARWIDTH 85
#define BARHEIGHT 9
#define BARBORDER 3
#define HEALTHBARX 82
#define HEALTHBARY 71
#define ARMORBARX 82
#define ARMORBARY 34

// This was supposed to be changeable from 
// within a config file...
DWORD dwHealthBg = 0xFF5E0B13;
DWORD dwHealth = 0xFFB8181A;
DWORD dwArmorBg = 0xFF727473;
DWORD dwArmor = 0xFFE3E5E2;


// Functions
void DrawRect(DWORD* pDestTexData, DWORD PixWidth, int x, int y, int width, int height, DWORD color)
{
	DWORD* pBuf = pDestTexData;

	pBuf += y*PixWidth+x;
	for (int ry=0;ry<height;ry++)
	{
		for (int rx=0;rx<width;rx++) 
		{
			*pBuf = color;
			pBuf++;
		}
	pBuf += PixWidth-width;
	}
}

void DrawDigit(DWORD* pDestTexData, DWORD PixWidth, int x, int y, int font, DWORD dwDigit)
{
	DWORD* pSrcBuf = pMiscHudTex;
	
	int spacing;
	int height;

	switch (font)
	{
	case 0:	spacing = 19;
			height = 27;
			break;
	case 1: pSrcBuf += 256*27;
			spacing = 19;
			height = 26;
			break;
	case 2: pSrcBuf += 256*53+2;
			spacing=10;
			height=11;
			break;
	}

	if (dwDigit == 10) pSrcBuf += 0;
	else pSrcBuf += ((dwDigit+1) * spacing);	// Move pointer to top-left pizel of the digit.

	pDestTexData += (y * PixWidth) + x;			// Set pointer to the right pixel of the texture.
	for (y = 0; y < height; y++)					// The digits are 32 pixels high.
	{
		memcpy(pDestTexData, pSrcBuf, spacing*4);		// Copy a row of pixels. The digits are 16 pixels wide, * 4bpp.
		pSrcBuf += 256;							// Skip to the next row of digit data.
		pDestTexData += PixWidth;				// Skip to next row of texture data.
	}
}

void DrawStar(DWORD* pDestTexData, DWORD PixWidth, int x, int y, bool wanted)
{
	DWORD* pSrcBuf = pMiscHudTex;

	pSrcBuf += 16640;
	if (wanted==false) pSrcBuf += 32;

	pDestTexData += (y * PixWidth) + x;
	for (y = 0; y < 32; y++)		
	{
		memcpy(pDestTexData, pSrcBuf, 128);	
		pSrcBuf += 256;
		pDestTexData += PixWidth;
	}
}

//--------------------------------------------------------------------------------
//	func:
//		CustomInitialize()
//
//	desc:
//		This is called when the game is first creating its Direct3D device or
//		when it has lost the device (happens when you tab out of full screen)
//		and has now successfully restored it.
//
//	params:
//		pDevice
//			This is a pointer to the real IDirect3DDevice8.
//
//		pPresentationParameters
//			This is the presentation parameters passed to the IDirect3DDevice8::Reset()
//			and IDirect3D8::CreateDevice().
//
//			Theres lots of useful infomation in the presentation params, but we only
//			need the screen size for the speedometer.
//
void CustomInitialize(IDirect3DDevice8* pDevice, D3DPRESENT_PARAMETERS* pPresentationParameters)
{
	fScreenWidth	= (float)pPresentationParameters->BackBufferWidth;		// Save screen width.
	fScreenHeight	= (float)pPresentationParameters->BackBufferHeight;		// Save screen height.

	if (!pHudTex) D3DXCreateTexture(pDevice, 256,128,0,0, D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &pHudTex);
	if (!pAmmoTex) D3DXCreateTexture(pDevice, 128,16,0,0, D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &pAmmoTex);
	if (!pWantedTex) D3DXCreateTexture(pDevice, 256,32,0,0, D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &pWantedTex);
	if (!pHudSprite) D3DXCreateSprite(pDevice, &pHudSprite);

	Dictionary TXD;
	LoadDict(&TXD,"SAHUD.TXD");

	// Loading weapon icons
	for (int i=0;i<34;i++) 
	{
		BYTE* Decompressed = NULL;
		Decompressed=(BYTE*)malloc((TXD.Textures[i].Header.Width * TXD.Textures[i].Header.Height) * 4);
		GetTexture(&TXD,i,Decompressed);

		D3DXCreateTexture(pDevice, TXD.Textures[i].Header.Width, TXD.Textures[i].Header.Height,0,0, D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &pWeaponsTex[i]);

		D3DLOCKED_RECT Rect;
		pWeaponsTex[i]->LockRect(0, &Rect, NULL, 0);
		BYTE* pSrcTexData = (BYTE*)Rect.pBits;
		memcpy(pSrcTexData,(void*)Decompressed,(TXD.Textures[i].Header.Width * TXD.Textures[i].Header.Height) * 4);
		pWeaponsTex[i]->UnlockRect(0);
	
		free(Decompressed);	
	}
	
	// Loading misc textures
	BYTE* Decompressed = NULL;
	Decompressed=(BYTE*)malloc((TXD.Textures[34].Header.Width * TXD.Textures[34].Header.Height) * 4);
	GetTexture(&TXD,34,Decompressed);

	D3DXCreateTexture(pDevice, TXD.Textures[34].Header.Width, TXD.Textures[34].Header.Height,0,0, D3DFMT_A8R8G8B8,D3DPOOL_MANAGED, &pMiscTex);

	D3DLOCKED_RECT Rect;
	pMiscTex->LockRect(0, &Rect, NULL, 0);
	BYTE* pSrcTexData = (BYTE*)Rect.pBits;
	memcpy(pSrcTexData,(void*)Decompressed,(TXD.Textures[34].Header.Width * TXD.Textures[34].Header.Height) * 4);
	pMiscHudTex = (DWORD*)Rect.pBits;
	pMiscTex->UnlockRect(0);

	free(Decompressed);

	vScaling.x = fScreenWidth / 1024.0f;
	vScaling.y = fScreenHeight / 768.0f;
	vScaling2.x = vScaling.x*1.0;
	vScaling2.y = vScaling.y*1.0;

	vTranslation.x = fScreenWidth-(230 * vScaling.x);	
	vTranslation.y = 37 * vScaling.y;

	vTranslationWanted.x = vTranslation.x;
	vTranslationWanted.y = vTranslation.y+(130*vScaling.y);

	vTranslationAmmo.x = vTranslation.x;
	vTranslationAmmo.y = vTranslation.y+(64*vScaling.y);

	// Version checking stuff.
	BYTE* VerCheck = (BYTE*)0x608578;
	if (*VerCheck == 0x81)	// Byte at 0x608578 = 0x81 in VC 1.1
	{
		// Modify pointers to work with Vice City v1.1...

		dwCanPlayRadio	= (DWORD*)(0x978670);
		bInMenu			= (BYTE*) (0x869670);
		dwPlrPointer	= (DWORD*)(0x94AD30);

		SCRIPT_BASE		= 0x821288;

		GameGetObject	= (GameGetObject_t)0x4BFFA0;
		GameGetVehicle	= (GameGetVehicle_t)0x4BFFE0;
		GameGetActor	= (GameGetActor_t)0x4C0020;
	}
	PLAYER_CHAR		= (DWORD*)(SCRIPT_BASE + 0x08);
	PLAYER_ACTOR	= (DWORD*)(SCRIPT_BASE + 0x0C);
}

//--------------------------------------------------------------------------------
//	func:
//		CustomRender()
//
//	desc:
//		This is called every time the game calls IDirect3DDevice8::Present(). The
//		game calls this function when it's done drawing all the game geometery and
//		wants to present (flip the back and front buffers) the screen to the user.
//
//		Our function is called before the real call to Present(), so it's the
//		perfect time todo some drawing :)
//
//	params:
//		pDevice
//			This is a pointer to the real IDirect3DDevice8.
//
void CustomRender(IDirect3DDevice8* pDevice)
{
	// Doesn't works :o
	BYTE* pHud=(BYTE*)0x95cf41;
	*pHud=0;

	// If those two are created, the others should be too
	if ((pHudSprite) && (pHudTex))
	{
		if ((*dwCanPlayRadio) && (!*bInMenu))	//
		{
			int nHealth, nHealthBar, nArmor, nArmorBar, nHours, nMinutes, nWantedLevel;
			int nClipAmmo, nAmmo;
			long nMoney;
			char cMoney[8], cHour[2], cMinute[2], cClipAmmo[6], cAmmo[6];	

			// Get Player health and armor
			ScriptCommand(&get_player_health, PLAYER_CHAR, &nHealth);
			if (nHealth>200) nHealth=200;
			nHealthBar = (int)(BARWIDTH*nHealth)/200;
			ScriptCommand(&get_player_armor, PLAYER_ACTOR, &nArmor);
			if (nArmor>200) nArmor=200;
			nArmorBar = (int)(BARWIDTH*nArmor)/200;

			// Get Player's money
			ScriptCommand(&get_player_money, PLAYER_CHAR, &nMoney);
			sprintf(cMoney,"%08u",nMoney);

			// Get wanted level
			ScriptCommand(&get_player_wanted_level, PLAYER_CHAR, &nWantedLevel);

			// Get current time
			ScriptCommand(&get_current_time, &nHours, &nMinutes);
			sprintf(cHour,"%02d",nHours);
			sprintf(cMinute,"%02d",nMinutes);

			// Weapon data
			BYTE* nWeaponIndex = (BYTE*)((*dwPlrPointer)+1284);
			BYTE* nWeaponData = (BYTE*)(((*dwPlrPointer)+1032)+((*nWeaponIndex)*24));
			DWORD* dwIndex = (DWORD*)nWeaponData;
			
			dwIndex+=2;
			nClipAmmo = sprintf(cClipAmmo,"%d",*dwIndex);
			DWORD dwAmmo = *dwIndex;
			dwAmmo *= -1;
			dwIndex+=1;
			dwAmmo += *dwIndex;
			nAmmo = sprintf(cAmmo,"%d",dwAmmo);
			dwIndex-=3;

			// Center text, really bad code I know
			vTranslationAmmo.x = vTranslation.x+(37*vScaling.x)-((int)((nAmmo+nClipAmmo+1)/2)*vScaling.x*10);
			
			// HUD
			D3DSURFACE_DESC Desc;
			pHudTex->GetLevelDesc(0, &Desc);	// Get the primary texture level description.
			if (Desc.Format == D3DFMT_A8R8G8B8)	// This should always be true, but blah.
			{
				D3DLOCKED_RECT Rect;
				pHudTex->LockRect(0, &Rect, NULL, 0);	// Lock the texture. Error checking here maybe? :

				DWORD* pTexData = (DWORD*)Rect.pBits;	// Pointer to the raw texture data.
				
				int PixRow = Rect.Pitch / 4;			// Pitch of the texture in pixels.
	
				DrawDigit(pTexData,PixRow,0,90,0,10);
				for (int i=0;i<8;i++) DrawDigit(pTexData, PixRow, 19+(19*i), 90, 0,((int)cMoney[i]-48));

				// Clock display
				DrawDigit(pTexData,PixRow,77,2,1,((int)cHour[0]-48));
				DrawDigit(pTexData,PixRow,96,2,1,((int)cHour[1]-48));
				DrawDigit(pTexData,PixRow,115,2,1,10);
				DrawDigit(pTexData,PixRow,134,2,1,((int)cMinute[0]-48));
				DrawDigit(pTexData,PixRow,153,2,1,((int)cMinute[1]-48));

				// Drawing the health bar
				DrawRect(pTexData,PixRow,(HEALTHBARX-BARBORDER),(HEALTHBARY-BARBORDER),BARWIDTH+(BARBORDER*2),(BARBORDER*2)+BARHEIGHT,0xFF000000);
				DrawRect(pTexData,PixRow,HEALTHBARX,HEALTHBARY,nHealthBar,BARHEIGHT,dwHealth);
				DrawRect(pTexData,PixRow,HEALTHBARX+nHealthBar,HEALTHBARY,BARWIDTH-nHealthBar,BARHEIGHT,dwHealthBg);

				// Drawing the armor bar
				if (nArmor>0) {
					DrawRect(pTexData,PixRow,(ARMORBARX-BARBORDER),(ARMORBARY-BARBORDER),BARWIDTH+(BARBORDER*2),(BARBORDER*2)+BARHEIGHT,0xFF000000);
					DrawRect(pTexData,PixRow,ARMORBARX,ARMORBARY,nArmorBar,BARHEIGHT,dwArmor);
					DrawRect(pTexData,PixRow,ARMORBARX+nArmorBar,ARMORBARY,BARWIDTH-nArmorBar,BARHEIGHT,dwArmorBg);
				} else 
					DrawRect(pTexData,PixRow,(ARMORBARX-BARBORDER),(ARMORBARY-BARBORDER),BARWIDTH+(BARBORDER*2),(BARBORDER*2)+BARHEIGHT,0x00000000);

				pHudTex->UnlockRect(0);
			}
			
			// WANTED LEVEL
			pWantedTex->GetLevelDesc(0, &Desc);	// Get the primary texture level description.
			if (Desc.Format == D3DFMT_A8R8G8B8)	// This should always be true, but blah.
			{
				D3DLOCKED_RECT Rect;
				pWantedTex->LockRect(0, &Rect, NULL, 0);

				DWORD* pTexData = (DWORD*)Rect.pBits;	
				
				int PixRow = Rect.Pitch / 4;

				int x=0;
				for (int i=0;i<6-nWantedLevel;i++)
				{
					DrawStar(pTexData,PixRow,x,0,false);
					x+=32;
				}
				for (i=6-nWantedLevel;i<6;i++)
				{
					DrawStar(pTexData,PixRow,x,0,true);
					x+=32;
				}
	
				pWantedTex->UnlockRect(0);
			}

			// AMMUNITION
			pAmmoTex->GetLevelDesc(0, &Desc);	// Get the primary texture level description.
			if (Desc.Format == D3DFMT_A8R8G8B8)	// This should always be true, but blah.
			{
				D3DLOCKED_RECT Rect;
				pAmmoTex->LockRect(0, &Rect, NULL, 0);

				DWORD* pTexData = (DWORD*)Rect.pBits;	
				
				int PixRow = Rect.Pitch / 4;

				DrawRect(pTexData,PixRow,0,0,127,15,0x00000000);

				for (int i=0;i<nClipAmmo;i++) DrawDigit(pTexData, PixRow, (10*i), 0, 2,(int)(cClipAmmo[i]-48));
				DrawDigit(pTexData, PixRow, 10*(nClipAmmo),0,2,10);
				for (i=0;i<nAmmo;i++) DrawDigit(pTexData, PixRow, 10*(nClipAmmo+1)+(10*i), 0, 2,(int)(cAmmo[i]-48));
				
				pAmmoTex->UnlockRect(0);
			}

			// Correcting ids for bomb
			DWORD dwTexIndex = *dwIndex;
			if (dwTexIndex == 13) dwTexIndex = 12; 
			if (dwTexIndex == 34) dwTexIndex = 13;
			
			// Draw all texes
			pHudSprite->Begin();
			pHudSprite->Draw(pHudTex, NULL, &vScaling, NULL, 0.0f, &vTranslation, 0xFFFFFFFF);
			if (nWantedLevel>0) 
				pHudSprite->Draw(pWantedTex, NULL, &vScaling, NULL, 0.0f, &vTranslationWanted, 0xFFFFFFFF);
			pHudSprite->Draw(pWeaponsTex[(int)(dwTexIndex)], NULL, &vScaling2, NULL, 0.0f, &vTranslation, 0xFFFFFFFF);
			// Weapons with ids above 11 require ammo
			if (*dwIndex>11) 
				pHudSprite->Draw(pAmmoTex, NULL, &vScaling, NULL, 0.0f, &vTranslationAmmo, 0xFFFFFFFF);
			
			pHudSprite->End();
		}
	}
}

//--------------------------------------------------------------------------------
//	func:
//		CustomReset()
//
//	desc:
//		This is called when the game calls IDirect3DDevice8::Reset().
//
//		The game calls this when it has lost the device and wants to regain it,
//		at this point you should release anything created from the device so it
//		can successfully reset.
//
//		If the device sucessfully resets, our CustomInitialize() function will
//		be called.
//
//	params:
//		n/a
//
void CustomReset()
{
	SAFE_RELEASE(pHudTex);
	SAFE_RELEASE(pWantedTex);
	SAFE_RELEASE(pAmmoTex);
	SAFE_RELEASE(pHudSprite);
	for (int i=0; i<34; i++) SAFE_RELEASE(pWeaponsTex[i]);
}